import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * <b>Der Monitor visualisiert das Weltmodell eines beliebigen Spielers</b>.
 * <br><br>
 * An der Kopfseite wird ein Kontainer mit 12 Buttons erstellt, welche
 * die Zahlen 1-11 und den Wert "Exit" enthalten. Mit einem click auf
 * eine der Zahlen kann ausgewhlt werden, welches Weltmodell man 
 * betrachten will. "Exit" beendet den Monitor, nicht jedoch das Programm.
 * <br><br>
 * Im Krper des Kontainers sieht man das Spielfeld, welches das
 * Weltmodell des gewhlten Spielers anzeigt. Die Objekte haben folgende
 * Farben:<br><br>
 *   Spieler selbst: rot<br>
 *   Mitspieler    : orange<br>
 *   Gegner        : grau<br>
 *   Unbekannt     : blau<br>
 *   Ball          : schwarz (klein)
 *
 */
public class Monitor extends Frame implements Runnable{
  /** @serial angezeigtes Weltbild */
  int spieler;
  /** @serial zuletzt angezeigtes Weltbild */
  int spieler1;
  /** @serial Kontainer mit dem Spielfeld */
  Feld spielfeld;
  /** @serial Referenz auf die Weltmodelle */
  Weltmodell welt[] = new Weltmodell[11];
  // Weltmodell alle 100 ms aktualisieren.
  /**
   * <b>Startet den Monitor-Thread</b>.
   */
  public void run() {
    for (;;) {
      try {
	Thread.sleep(100);
	}
      catch(InterruptedException e) {}
      this.spielfeld.repaint();
      }
    }
  /**
   * <b>Konstruktor des Monitors</b>.
   * <br><br>
   * Die Variable <i>welt</i> mu ein Array vom Typ Weltmodell sein.
   */
  public Monitor(Weltmodell[] welt) {
    super("Weltbild");
    this.welt = welt;
    spieler=1;
    spieler1=1;
    // Kontainer fr Kopfseite und Krper
    setLayout(new BorderLayout());
    // Kopfseite fr Auswahlleiste
    Panel panel = new Panel();
    panel.setLayout(new GridLayout(1,12));
    Button player1= new Button("1");
    Button player2= new Button("2");
    Button player3= new Button("3");
    Button player4= new Button("4");
    Button player5= new Button("5");
    Button player6= new Button("6");
    Button player7= new Button("7");
    Button player8= new Button("8");
    Button player9= new Button("9");
    Button player10= new Button("10");
    Button player11= new Button("11");
    Button ende= new Button("Exit");
    panel.add(player1);
    panel.add(player2);
    panel.add(player3);
    panel.add(player4);
    panel.add(player5);
    panel.add(player6);
    panel.add(player7);
    panel.add(player8);
    panel.add(player9);
    panel.add(player10);
    panel.add(player11);
    panel.add(ende);
    add("North", panel);

    // Horcht, ob auf das Fenster-"X" geclickt wird.
    class Mein_WindowAdapter extends WindowAdapter {
      public void windowClosing(WindowEvent e) {
        hide();
      }
    }

    // Horcht, welcher Button gedrckt wird
    class Mein_ActionListener implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	String aufruf = e.getActionCommand();
        if (aufruf.equals("Exit")) {
            hide();
          }
        else { int ruf = Integer.decode(aufruf).intValue();
	       spieler=ruf;
	       spielfeld.repaint();
	     }
      }
    }

    player1.addActionListener(new Mein_ActionListener());
    player2.addActionListener(new Mein_ActionListener());
    player3.addActionListener(new Mein_ActionListener());
    player4.addActionListener(new Mein_ActionListener());
    player5.addActionListener(new Mein_ActionListener());
    player6.addActionListener(new Mein_ActionListener());
    player7.addActionListener(new Mein_ActionListener());
    player8.addActionListener(new Mein_ActionListener());
    player9.addActionListener(new Mein_ActionListener());
    player10.addActionListener(new Mein_ActionListener());
    player11.addActionListener(new Mein_ActionListener());
    ende.addActionListener(new Mein_ActionListener());
    addWindowListener(new Mein_WindowAdapter());
    // Krper fr Spielfeld
    spielfeld = new Feld();
    add("Center", spielfeld);
    pack();
    setSize(570,440);
    setResizable(false);
    show();
  }
    
  /**
   * <b>Ein Kontainer, in dem das Spielfeld eingezeichnet wird</b>.
   */
  public class Feld extends Canvas {
    Feld() {
      setBackground(Color.green);
      setForeground(Color.white);
      }
    /**
     * <b>Zeichnet den Kontainer und die Objekte</b>.
     * <br><br>
     * Die Methode berschreibt die Methode <i>paint(Graphics g)</i> der Klasse Canvas.<br>
     * Sie zeichnet das Spielfeld und ruft die Methode <i>drawObjecs(Graphics g)</i> auf,
     * welche die Spieler und den Ball einzeichnet.
     */
    public void paint(Graphics g) {
      if (welt[spieler-1]==null) { spieler=spieler1; }
      else { spieler1=spieler; }
      // Spielfeld zeichnen
      g.translate(285,190);
      g.drawLine(-262,-170,262,-170);
      g.drawLine(-262,-170,-262,170);
      g.drawLine(-262,170,262,170);
      g.drawLine(262,-170,262,170);
      g.drawLine(0,-170,0,170);
      g.drawLine(-180,-101,-180,101);
      g.drawLine(180,-101,180,101);
      g.drawLine(-262,-101,-180,-101);
      g.drawLine(-262,101,-180,101);
      g.drawOval(-45,-45,90,90);
      g.drawLine(262,-101,180,-101);
      g.drawLine(262,101,180,101);
      g.drawLine(210,0,210,0);
      g.drawLine(-210,0,-210,0);
      g.drawLine(-235,-45,-235,45);
      g.drawLine(235,-45,235,45);
      g.drawLine(-262,-45,-235,-45);
      g.drawLine(-262,45,-235,45);
      g.drawLine(262,-45,235,-45);
      g.drawLine(262,45,235,45);
      // Objekte Zeichnen
      drawObjects(g);
      }

    /**
     * <b>Zeichnet die Spieler und den Ball</b>.
     */
    public void drawObjects(Graphics g) {
      // Ball
      if (welt[spieler-1].object[0]!=null)
	{
	g.setColor(Color.black);
	g.fillOval( (int) (welt[spieler-1].object[0].x*5-3)  , (int) (welt[spieler-1].object[0].y*5-3) , 6, 6);
	}
      // Selbst
      if (welt[spieler-1].object[1]!=null) 
	{
	g.setColor(Color.red);
	g.fillOval( (int) (welt[spieler-1].object[1].x*5-5)  , (int) (welt[spieler-1].object[1].y*5-5) ,10,10);
        }
      // Andere Spieler
      int obj=2;
      while (obj<23 && welt[spieler-1].object[obj]!=null) 
	{
        // Unbekannt
	g.setColor(Color.blue);
        // Mitspieler
	if (welt[spieler-1].object[obj].team == 1) { g.setColor(Color.orange); }
        // Gegner
	if (welt[spieler-1].object[obj].team == -1) { g.setColor(Color.darkGray); }
        // Einzeichnen
	g.fillOval( (int) (welt[spieler-1].object[obj].x*5-5)  , (int) (welt[spieler-1].object[obj].y*5-5) ,10,10);
	obj++;
        }
      }

    /**
     * Legt die Mindestgre des Fensters fest.
     */
    public Dimension getMinimumSize() {
      return new Dimension(570,380);
      }

    /**
     * Setzt die Fenstergre auf Mindestgre.
     */
    public Dimension getPreferredSize() {
      return getMinimumSize();
      }
    }
  }
