import java.io.*;
import java.net.*;

/**
 * <b>Dies ist die Netzwerk-Komponente des Programms</b>.
 * <br><br>
 * Sie meldet zuerst den Spieler beim Server an, wartet dann auf die
 * Meldungen des Servers und steuert die Reaktion darauf.
 * <br><br>
 * Im Falle einer See-Nachricht wird diese an das Weltmodell geliefert.
 * <br><br>
 * Hear-Nachrichten vom Referee ndern den Spielstatus auf die Werte
 * play_on, free_kick_l, free_kick_r oder before_kick_off.
 * Hear-Nachrichten von Mitspielern sollen genutzt werden, um das
 * Weltbild des Spielers zu vergrern. Diese Funktion ist jedoch noch
 * nicht ausreichend implementiert, so da sie momentan noch zu sehr
 * fr Strungen sorgt.
 * <br><br>
 * Die Init-Nachricht wird direkt von der Netzwerk-Komponente verarbeitet.
 * Dabei werden die Spieler auf ihre Startposition gesetzt.
 * <br><br>
 * Sense_Body-Nachrichten werden zur Taktung genutzt, da sie im selben
 * Zeitabstand getaktet sind wie Befehle an den Server gesendet werden
 * knnen. Die Nachrichten werden in den Klassen Torwart, Abwehr,
 * Mittelfeld und Sturm verarbeitet, welche von der Klasse Spieler
 * abgeleitet sind.
 *
 */
public class Net extends Thread {
  int FreeKickCycle;
  String status;
  DatagramSocket socket;
  InetAddress serverAddress;
  int serverPort;
  Weltmodell welt;
  Spieler player;
  String teamname;
  /**
   * <b>Konstruktor der Klasse Net</b>.
   * <br><br>
   * serverName=Adresse des Servers<br>
   * serverPort=Port des Servers<br>
   * w=Weltmodell des Spielers<br>
   * p=Spieler<br>
   * t=Mannschaftsname des Spielers<br>
   */
  public Net(String serverName, int serverPort, Weltmodell w, Spieler p, String t) {
    try {
      this.socket        = new DatagramSocket();
      this.serverAddress = InetAddress.getByName(serverName);
      this.serverPort    = serverPort;
      }
    catch (SocketException se) {
      System.err.println("CommunicationChannel: Could not create datagram socket.");
      System.exit(1);
      }
    catch (UnknownHostException uhe) {
      System.err.println("CommunicationChannel: Could not determine the server's IP address.");
      System.exit(1);
      }
    welt=w;
    player=p;
    teamname=t;
    FreeKickCycle=-100;
    status="before_kick_off";
    }
  /**
   * <b>run-Methode des Threads</b>.
   */
  public void run() {
    // Torwart
    if (player.number==1) { send("(init "+teamname+" (version 7.0) (goalie))"); }
    // Feldspieler
    else { send("(init "+teamname+" (version 7.0))"); }
    for(;;) {
      // Nachricht empfangen
      String info=receive();
      // See-Nachricht
      if (info.charAt(3)=='e') { welt.getSeeFromNet(info); }
      // Hear-Nachricht
      if (info.charAt(1)=='h') {
	String message = info.substring(info.indexOf(" ")+1);
        message = message.substring(message.indexOf(" ")+1);
	String sender=message.substring(0, message.indexOf(" "));
        // vom Referee
	if (sender.equals("referee")) {
          message = message.substring(0, message.indexOf(")"));
	  message=message.substring(message.indexOf(" ")+1);
	  status=message;
          // Status aktualisieren
          if (status.startsWith("goal_k"))      { status = "free_kick_"+status.charAt(status.length()-1); }
          if (status.startsWith("goal"))      { status = "before_kick_off"; welt.object[0]=null; welt.object[0]=new MovableObject();}
          if (status.startsWith("kick_off")) { status = "play_on"; }
          if (status.startsWith("drop"))      { status = "play_on"; }
          if (status.startsWith("kick_in"))  { status = "free_kick_"+status.charAt(status.length()-1); }
          if (status.startsWith("corner"))   { status = "free_kick_"+status.charAt(status.length()-1); }
          if (status.startsWith("offside"))   { 
            if (status.charAt(status.length()-1)=='l') status = "free_kick_r";
	    else status = "free_kick_l";
	    }
          if (status.startsWith("free")) {
            String zeit = info.substring(6);
            zeit=zeit.substring(0, zeit.indexOf(" "));
            FreeKickCycle=Integer.valueOf(zeit).intValue();
            }
          if (!status.startsWith("play")) {
	    welt.object[0]=null;
	    }
	  }
        // vom Mitspieler
//	if ((sender.equals("referee")||sender.equals("online_coach_left")||sender.equals("online_coach_right")||sender.equals("self"))==false) {
//	  message = message.substring(message.indexOf('"')+1, message.indexOf(")")-1);
//	  welt.getHearFromNet(message);
//	  }
	}
      // Init-Nachricht
      if (info.charAt(1)=='i') {
        // Standartmig von Links nach Rechts, wenn nicht wird hier
        // die Richtung gendert.
        if (info.charAt(6)=='r') {
          welt.goalDirection=-1;
          }
        // Spieler an Startposition stellen
        send("(move "+player.homeX+" "+player.homeY+")");
        }
      // Sense_Body-Nachricht
      if (info.charAt(3)=='n') {
        info=info.substring(12);
        // Vom Spieler die Reaktion anfordern
        String decision = player.makeDecision(info);
        send(decision);
        }
      }
    }

  /**
   * <b>Schickt eine String-Nachrichten an den Server</b>.
   */
  public void send(String message) {
    try {
      byte[] buffer = message.getBytes();
      DatagramPacket packet = new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
      socket.send(packet);
      }
    catch (IOException ioe) {
      System.err.println("CommunicationChannel: Could not send datagram packet.");
      }
    }

  /**
   * <b>Empfngt eine String-Nachrichten vom Server</b>.
   */
  public String receive() {
    byte[] buffer = new byte[3072];
    DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
    try {
      socket.receive(packet);
      }
    catch (IOException ioe) {
      System.err.println("CommmunicationChannel: Could not receive datagram packet.");
      }
    serverPort = packet.getPort();
    return new String(packet.getData()).substring(0, packet.getLength()-1)+" ";  // cut off null characters & \n in buffer
    }
}

