/*
 Das Programm RoboClient wurde im Praktikum Programmiermethodik an der
 Universitt Mannheim erstellt. Das Programm bietet einen Client fr
 die Simulation League das Projektes "RoboCup", welche die Strategie
 Komponente einer Fussball-spielenden Sony-Roboter-Mannschaft bildet.
 */

import java.io.*;

/**
 * <b>Hauptklasse des Projekts RoboClient</b>.
 * <br><br>
 * Das Programm wird mit dem Aufruf
 * <br><br>
 * 	java RoboClient TEAMNAME [SERVERNAME SERVERPORT] [+M]
 * <br><br>
 * gestartet. Der Wert TEAMNAME kann beliebig gewhlt werden. Die
 * Argumente SERVERNAME und SERVERPORT sind gemeinsam optional, sie haben
 * die Standartwerte "localhost" und "6000".
 * <br><br>
 * Das Programm bietet zustzlich einen Monitor, der das aktuelle Weltbild
 * eines beliebigen Spielers der Mannschaft visualiesiert. Dieser wieder
 * gestartet, wenn der Programmaufruf die Option "+M" besitzt.
 * <br><br>
 * Der Monitor kann entweder ber den "X"-Button des Fensters oder ber den
 * "Exit"-Button beendet werden.<br>
 * Das Programm meldet sich mit dem Befehl "exit" ordnungsgem beim Server
 * ab und beendet das Programm.
 * <br><br>
 * Die Klasse RoboClient initialisiert das Programm.<br>
 * Sie verarbeitet zuerst den Programmaufruf, inizialisiert dann elf Weltmodelle,
 * Spieler und Netzwerkkomponenten. Da das Programm nur auf Meldungen vom
 * Server reagiert wurde die Netzwerkkomponente als ein Thread erstellt,
 * welcher das Programm steuert.<br>
 * Schlielich wird der Monitor gestartet.
 */
public class RoboClient {
  /**
   * <b>Main-Methode der Klasse RoboClient</b>.
   */
  public static void main(String[] args) {
    boolean vis=false;
    String teamname = "Team";
    String host="localhost";
    int port=6000;
    // Programmaufruf verarbeiten
    if (args.length==1) {
      teamname = args[0];
      }
    if (args.length==2) {
      if ((args[1].equals("+m")==false)&&(args[1].equals("+M")==false)) { wrongCommand(); }
      else {
        teamname = args[0];
        vis=true;
        }
      }
    if (args.length==3) {
      teamname = args[0];
      host=args[1];
      try {
        port=Integer.valueOf(args[2]).intValue();
        }
      catch(NumberFormatException nfe) {
        System.out.println("Der Serverport muss eine Ganzzahl sein!");
        wrongCommand();
        }
      }
    if (args.length==4) {
      teamname = args[0];
      host=args[1];
      try {
        port=Integer.valueOf(args[2]).intValue();
        }
      catch(NumberFormatException nfe) {
        System.out.println("Der Serverport muss eine Ganzzahl sein!");
        wrongCommand();
        }
      if ((args[3].equals("+m")==false)&&(args[3].equals("+M")==false)) { wrongCommand(); }
      vis=true;
    }
    // Initialisierung der Komponenten
    Weltmodell[] welt=new Weltmodell[11];
    Spieler player[]=new Spieler[11];
    Net netz[]=new Net[11];
    for (int i=0; i<11; i++) {
      welt[i]=new Weltmodell(teamname, i+1);
      }
    player[0]=new Torwart(1, welt[0]);
    player[1]=new Abwehr(2, welt[1]);
    player[2]=new Abwehr(3, welt[2]);
    player[3]=new Abwehr(4, welt[3]);
    player[4]=new Abwehr(5, welt[4]);
    player[5]=new Mittelfeld(6, welt[5]);
    player[6]=new Mittelfeld(7, welt[6]);
    player[7]=new Mittelfeld(8, welt[7]);
    player[8]=new Sturm(9, welt[8]);
    player[9]=new Sturm(10, welt[9]);
    player[10]=new Sturm(11, welt[10]);
    // Starten der Netzwerkkomponenten
    for (int i=0; i<11; i++) {
      netz[i]=new Net(host, port, welt[i], player[i], teamname);
      player[i].netz=netz[i];
      netz[i].start();
      }
    // Monitor starten
    if (vis) {
      Runnable monitor = new Monitor(welt);
      Thread mon=new Thread(monitor);
      mon.start();
      }
    // Eingabe verarbeiten, bei bedarf das Programm beenden.
    BufferedReader eingabe = new BufferedReader(new InputStreamReader(System.in));
    for (;;) {
      try {
        String aufruf=eingabe.readLine();
        if (aufruf.toLowerCase().equals("exit")) {
          bye(netz);
          }
      }
      catch (IOException ioe) {}
      }
    }

  /** <b>Meldet das Programm vom Server ab und beendet es</b>. */
  public static void bye(Net[] netz) {
    for (int i=0; i<11; i++) {
      netz[i].send("(bye)");
      }
    System.exit(0);
    }

  /** <b>Gibt die Rckmeldung bei falschem Programmaufruf aus</b>. */
  public static void wrongCommand() {
      System.out.println("****************************************************************");
      System.out.println("Falscher Programmaufruf:");
      System.out.println();
      System.out.println("java RoboClient TEAMNAME [SERVERNAME SERVERPORT] [+M]");
      System.out.println();
      System.out.println("TEAMNAME                        (String): Name der Mannschaft");
      System.out.println("SERVERNAME ('localhost' oder IP-Adresse): Adresse des RCSSServer");
      System.out.println("SERVERPORT                    (Ganzzahl): Port des RCSSServer");
      System.out.println("+M                                : Monitor einschalten");
      System.out.println("****************************************************************");
      System.out.println();
      System.exit(1);
      }
  }



