/**
 * <b>Basisklasse fr die Entscheidungen</b>.
 * <br><br>
 * Die Klasse Spieler bietet Hilfsmethoden, die zur Entscheidungsfindung
 * beitragen. Die Klassen Torwart, Abwehr, Mittelfeld und Sturm werden
 * von der Klasse Spieler abgeleitet.
 */
public class Spieler {
  int cycle;
  double stamina;
  int criticalAngle;
  /** Rckennummer des Spielers */
  public int number;
  /** Ursprungs-X-Koordinate des Spielers */
  public int areaX;
  /** Ursprungs-Y-Koordinate des Spielers */
  public int areaY;
  /** Start-X-Koordinate des Spielers */
  public int homeX;
  /** Start-Y-Koordinate des Spielers */
  public int homeY;
  boolean aussen;
  /** Weltmodell des Spielers */
  public Weltmodell welt;
  /** Netzwerkkomponente des Spielers */
  public Net netz;
  /** Entfernungen der anderen Objekte zum Ball */
  public double[] ToBall = new double[23];
  /** Position des Ball-Nchsten Spieler im Array <i>object</i> */
  public int nearestPlayer;
  /** Position des Ball-Nchsten Mitspielers im Array <i>object</i> */
  public int nearestTeamPlayer;
  /** X-Koordinate, wo der nchste Spieler an den Ball kommt. */
  public double nearestX;
  /** Y-Koordinate, wo der nchste Spieler an den Ball kommt. */
  public double nearestY;
  /**
   * <b>Konstruktor der Klasse Spieler</b>.
   * <br><br>
   * n=Rckennummer des Spielers<br>
   * w=Weltmodell des Spielers
   */
  public Spieler(int n, Weltmodell welt) {
    this.welt=welt;
    number=n;
    cycle=0;
    criticalAngle=4;
    switch(number) {
      case 1: aussen=false; areaX=-51; areaY=0; homeX=-51; homeY=0; break;
      case 2: aussen=true; areaX=-15; areaY=-20; homeX=-15; homeY=-20; break;
      case 3: aussen=false; areaX=-25; areaY=-5; homeX=-25; homeY=-5; break;
      case 4: aussen=false; areaX=-25; areaY=5; homeX=-25; homeY=5; break;
      case 5: aussen=true; areaX=-15; areaY=20; homeX=-15; homeY=20; break;
      case 6: aussen=true; areaX=10; areaY=-20; homeX=0; homeY=-20; break;
      case 7: aussen=false; areaX=0; areaY=0; homeX=-10; homeY=-1; break;
      case 8: aussen=true; areaX=10; areaY=20; homeX=0; homeY=20; break;
      case 9: aussen=false; areaX=30; areaY=0; homeX=-10; homeY=1; break;
      case 10: aussen=true; areaX=40; areaY=-15; homeX=-1; homeY=0; break;
      case 11: aussen=true; areaX=40; areaY=15; homeX=0; homeY=20; break;
      }
    }

  /**
   * <b>Bestimmt, welche Entscheidung getroffen werden soll</b>.
   * <br><br>
   * Ist die Ballposition unbekannt, dreht sich der Spieler um 80 Grad.<br>
   * Ist die letzte Feststellung der Ballposition zu lange her wird die
   * Methode <i>turnToBall()</i> aufgerufen.<br>
   * Ist der Spielstatus <i>before_kick_off</i> wird der Spieler an seinen
   * Startpunkt gestellt.<br>
   * Ist der Spielstatus <i>play_on</i> wird die Methode <i>play()</i> aufgerufen.
   * Diese sollte aber in den Positions-Klassen berschrieben werden.<br>
   * Ist der Spielstatus <i>free_kick_l</i> bzw. <i>free_kick_r</i> so wird die
   * Methode <i>ownFree()</i> bzw. <i>opponentFree</i> aufgerufen.
   * Diese sollte aber in den Positions-Klassen berschrieben werden.
   */
  public String makeDecision(String info) {
    if (netz.status.equals("before_kick_off")) {
      if ((welt.object[1]==null)||((welt.object[1]!=null)&&(distance(1, homeX, homeY)>5)))
        return("(move "+homeX+" "+homeY+")");
      if ((number==10)&&(Math.abs(welt.object[1].direction)>2)) return("(turn "+(0-welt.object[1].direction)+")");
      }
    if (welt.object[0]==null) { 
      welt.object[1].direction = welt.object[1].direction+80;
      if (welt.object[1].direction>180) welt.object[1].direction=welt.object[1].direction-360;
      return("(turn 80)");
      }
    cycle=Integer.valueOf(info.substring(0, info.indexOf(" "))).intValue();
    if (cycle-welt.object[0].time1+2>welt.criticalTime) { return(turnToBall()); }
    info=info.substring(info.indexOf("stamina")+8);
    stamina=Double.valueOf(info.substring(0, info.indexOf(" "))).doubleValue();
    toBall();
    if (netz.status.equals("play_on")) return(play());
    if (netz.status.startsWith("free")) {
      if (netz.status.charAt(netz.status.length()-1)=='l') {
        if (welt.goalDirection==1) return(ownFree());
        else return(opponentFree());
        }
      if (netz.status.charAt(netz.status.length()-1)=='r') {
        if (welt.goalDirection==1) return(opponentFree());
        else return(ownFree());
        }
      }
    return("");
    }

/* ************ werden berschrieben ************************ */
  /** <b>Entscheidung bei eigenem Freisto (Muss berschrieben werden)</b> */
  public String ownFree() { return(""); }
  /** <b>Entscheidung bei gegnerischem Freisto (Muss berschrieben werden)</b> */
  public String opponentFree() { return(""); }
  /** <b>Entscheidung bei laufendem Spiel (Muss berschrieben werden)</b> */
  public String play() { return(""); }
/* ********************************************************** */

  /**
   * <b>Mildet mit dem Spieler eine Mauer<b>.
   */
  public String mauer() {
    return moveTo(-52.5+(52.5+welt.object[0].x)*(distance(0,24)-9)/distance(0,24), welt.object[0].y*(distance(0, 24)-9)/distance(0,24));
    }

  /**
   * <b>Spieler deckt einen Gegner</b>.
   * <br><br>
   * Der Spieler sucht nach dem nchsten Gegner in der Region 20m vor und hinter,
   * 15m seitlich von seinem Ursprungspunkt und rennt zu diesem Spieler. Existiert
   * kein solcher Gegner rennt er zu seinem Ursprungspunkt.
   */
  public String decken() {
    int spieler=2;
    int opfer=0;
    double opferDist=100;
    while ((spieler<23)&&(welt.object[spieler]!=null)) {
      if ((welt.object[spieler].team!=1)&&(Math.abs(welt.object[spieler].x-areaX)<20)&&(Math.abs(welt.object[spieler].y-areaY)<15)&&(distance(1,spieler)<opferDist)) {
        opfer=spieler;
        opferDist=distance(1, spieler);
        }
      spieler++;
      }
    if (opfer==0) return(moveTo(areaX, areaY));
    if (distance(1, opfer)<4) return(turnToBall());
    return(moveTo(welt.object[opfer].x+2, welt.object[opfer].y));
    }

  /**
   * <b>Steuert das Abwehrverhalten eines Abwehr- und Mittelfeldspielers</b>.
   * <br><br>
   * Spielt der Spieler an einer Aussenposition wird die Methode <i>decken()</i>
   * aufgerufen.<br>
   * Ist dies nicht der Fall und ist er dem Ball am nchsten aus dem eigenen Team,
   * so rennt er in Richtung Ball. Ansonsten rennt er in die Richtung der Linie
   * zwischen Ball und Tormittelpunkt.
   */
  public String defense() {
    if (distance(1, 0)<0.7) return(offense());
    if (ToBall[1]<ToBall[nearestTeamPlayer]+5) return(moveToBall());
    if (aussen||(welt.object[0].x>0)) return(decken());
    double winkel2=0;
    if (welt.object[0].y+welt.object[0].y*(welt.object[1].x+52.5)/(welt.object[0].x+52.5)>welt.object[1].y+2) winkel2=direction(0, -52.5, 0)-90;
    if (welt.object[0].y+welt.object[0].y*(welt.object[1].x+52.5)/(welt.object[0].x+52.5)<welt.object[1].y-2) winkel2=direction(0, -52.5, 0)+90;
    if (winkel2==0) {
      if(welt.object[1].x<welt.object[0].x) return(turnToBall());
      else return(moveToBall());
      }
    if ((winkel2>0)&&(direction(1,0)>winkel2)) return(moveToBall());
    if ((winkel2<0)&&(direction(1,0)<winkel2)) return(moveToBall());
    winkel2=winkel2-welt.object[1].direction;
    if (winkel2>180) winkel2=winkel2-360;
    if (winkel2<-180) winkel2=winkel2+360;
    if (Math.abs(winkel2)>20) return("(turn "+winkel2+")");
    return("(dash 100)");
    }

  /**
   * <b>Der Spieler versucht in eine mglichst gut anspielbare Position zu gelangen</b>.
   */
  public String freilaufen() {
    if (((stamina<2000)||((welt.object[0].x<welt.object[1].x)&&(2.7*distance(1, areaX, areaY)<distance(0, areaX, areaY))))&&(cycle%3!=0)) return(turnToBall());
    double optx=areaX;
    double opty=areaY;
    if ((number<9)&&aussen) { optx=welt.object[0].x+(52.5-welt.object[0].x)/2; }
    if ((number>8)&&aussen) {
      optx = welt.object[0].x+(52.5-welt.object[0].x)*2/3;
      }
    if (optx<areaX) optx=areaX;
    return(moveTo(optx, opty));
    }

  /** <b>Der Spieler passt den Ball in Richtung Zielobjekt</b>. */
  public String Pass(int ziel) {
    double angle = direction(1, ziel)-welt.object[1].direction;
    if (angle > 180) angle=angle-360;
    if (angle<-180) angle=angle+180;
    double force = 3.5*distance(1,ziel);
    if (force>100) force=100;
    welt.object[0].xspeed=0.027*force*Math.cos(Math.toRadians(angle));
    welt.object[0].yspeed=0.027*force*Math.sin(Math.toRadians(angle));
    return ("(kick "+force+" "+angle+")");
    }

  /**
   * <b>Steuert das Offensiv-Verhalten eines Mittelfeld-Spielers und eines Strmers</b>.
   * <br><br>
   * Ist der Ball in Schussreichweite und der Weg zum Tor frei, so
   * wird in Richtung Tor geschossen. Ist der Weg versperrt wird nach
   * einem freien Mitspieler gesucht, dem gepasst werden kann.
   * Findet sich kein freier Mitspieler und hat der Spieler einen gewissen
   * Freiraum um sich herum, so macht er einen leichten Kick in Richtung
   * Tor, ansonsten schiesst er Richtung Tor.
   */
  public String offense() {
    if (distance(1, 0)<0.7) {
      if (ToBall[1]>0.7) return(turnToBall());
      double angle = direction(0, 23);
      if ((welt.object[0].x>30)||(distance(0,23)<30)){
        angle=angle-welt.object[1].direction;
        if (angle>180) angle = angle - 360;
        if (angle<-180) angle= angle+360;
        if (Math.abs(angle)>30) return("(turn "+angle+")");
        welt.object[0].xspeed=2.7*Math.cos(Math.toRadians(angle));
        welt.object[0].yspeed=2.7*Math.sin(Math.toRadians(angle));
        return("(kick 100 "+angle+")");
        }
      int ziel = findPlayerToPass();
      if (ziel != 0) return(Pass(ziel));
      int counter=2;
      double Freiraum=100;
      while ((counter<23)&&(welt.object[counter]!=null)) {
        if (((welt.object[1].x<welt.object[counter].x)||(distance(1,counter)<5))&&(distance(1,counter)<Freiraum)) Freiraum=distance(1,counter);
        counter++;
        }
      if ((Freiraum>15)&&(Math.abs(welt.object[1].direction)<70)) {
	if ((welt.object[1].y>0)&&(welt.object[1].direction<0)&&(welt.object[1].direction>-45)) { angle=welt.object[1].direction; }
	if ((welt.object[1].y<0)&&(welt.object[1].direction>0)&&(welt.object[1].direction<45)) { angle=welt.object[1].direction; }
        welt.object[0].xspeed=0.8*Math.cos(Math.toRadians(angle));
        welt.object[0].yspeed=0.8*Math.sin(Math.toRadians(angle));
        angle=angle-welt.object[1].direction;
        if (angle>180) angle = angle - 360;
        if (angle<-180) angle= angle + 360;
        return("(kick 20 "+angle+")");
        }
      angle=direction(0,23)-welt.object[1].direction;
      double d=distance(0,23);
      if (welt.object[0].y>0) {
        if (welt.object[0].x<0) {
          angle=direction(0, 40, 15)-welt.object[1].direction;
          d=distance(0, 40, 15);
          }
        if (welt.object[0].x<-15) {
          angle=direction(0, 10, 20)-welt.object[1].direction;
          d=distance(0, 10, 20);
          }
        }
      else {
        if (welt.object[0].x<0) {
          angle=direction(0, 40, -15)-welt.object[1].direction;
          d=distance(0, 40, -15);
          }
        if (welt.object[0].x<-15) {
          angle=direction(0, 10, -20)-welt.object[1].direction;
          d=distance(0, 10, -20);
          }
        }
      d=Math.min(100, d*7);
      if (angle>180) angle = angle - 360;
      if (angle<-180) angle = angle + 360;
      return("(kick "+(int)(d)+" "+angle+")");
      }
    if (nearestTeamPlayer!=1) return(freilaufen());
    return(moveToBall());
    }

  /**
   * <b>Dreht den Spieler in Richtung der zuletzt bekannten Ballposition</b>.
   */
  public String turnToBall() {
    double angle=direction(1, 0)-welt.object[1].direction;
     if ((welt.object[1].direction)+angle>180) {
      welt.object[1].direction=welt.object[1].direction+angle-360;
      return("(turn "+angle+")");
      }
    if ((welt.object[1].direction+angle)<-180) {
      welt.object[1].direction=welt.object[1].direction+angle+360;
      return("(turn "+angle+")");
      }
    welt.object[1].direction=welt.object[1].direction+angle;
    return("(turn "+angle+")");
    }

  /*
   * <b>Berechnet den optimalen Weg zum Ball</b>.
   */
  public String moveToBall() {
    double winkel = direction(0, 1)-direction(0, welt.object[0].x+welt.object[0].xspeed,  welt.object[0].y+welt.object[0].yspeed);
    if (winkel>180) winkel = winkel-360;
    if (winkel<-180) winkel = winkel+360;
    if (((nearestTeamPlayer!=1)||(winkel>100)) && (number!=1)) return(moveTo(welt.object[0].x+welt.object[0].xspeed, welt.object[0].y+welt.object[0].yspeed));
    int zeit=0;
    double tempxspeed=welt.object[0].xspeed;
    double tempyspeed=welt.object[0].yspeed;
    double tempx=welt.object[0].x;
    double tempy=welt.object[0].y;
    boolean unreached=true;
    while (unreached) {
      tempxspeed=tempxspeed*0.97;
      tempx=tempx+tempxspeed;
      tempyspeed=tempyspeed*0.97;
      tempy=tempy+tempxspeed;
      zeit++;
      if (distance(1, tempx, tempy)<zeit) {
        double dir=direction(1, tempx, tempy)-welt.object[1].direction;
        if (dir>180) dir = dir-360;
        if (dir<-180) dir = dir+360;
        if ((Math.abs(dir)<20)||(distance(1, tempx, tempy)<zeit-1)) unreached = false;
        }
      }
    return(moveTo(tempx, tempy));
    }

  /**
   * <b>Lsst den Spieler zu einem bestimmten Punkt laufen</b>.
   * <br><br>
   * Ist die Entfernung zum Punkt (optx/opty) kleiner als 0.7 macht
   * der Spieler gar nichts.<br>
   * Ist der Winkel zu diesem Punkt grer als 20 Grad dreht sich der
   * Spieler in Richtung Punkt, ansonsten beschleunigt der Spieler.
   */
  public String moveTo(double optx, double opty) {
    if ((ToBall[1]>20)&&(distance(1, optx, opty)<2)) return(turnToBall());
    if (opty>35.5) opty = 35.5;
    if (opty<-35.5) opty = -35.5;
    if (optx>53) optx = 53;
    if (optx<-53) optx = -53;
    // Abseits
    if (optx>0) {
      int vorne1=24; 
      int vorne2=24;
      int spieler=2;
      while ((spieler<23)&&(welt.object[spieler]!=null)) {
        if ((welt.object[spieler].team!=1)&&(welt.object[spieler].x>welt.object[vorne1].x)) vorne1=spieler;
        if (welt.object[vorne1].x>welt.object[vorne2].x) {
          int temp=vorne2;
          vorne2=vorne1;
          vorne1=temp;
          }
        spieler++;
        }
      if ((optx>welt.object[vorne1].x)&&(optx>welt.object[0].x+welt.object[0].xspeed)&&(nearestTeamPlayer!=1)) {
        optx=welt.object[vorne1].x;
        if (optx<0) optx=0;
        }
      }
    double richtung =direction(1, optx, opty)-welt.object[1].direction;
    if (richtung>180) richtung = richtung-360;
    if (richtung<-180) richtung = richtung+360;
    if ((Math.abs(richtung)>20)&&(Math.abs(richtung)<160)) {
      if ((welt.object[1].direction+richtung)>180) {
        welt.object[1].direction=welt.object[1].direction+richtung-360;
        return("(turn "+richtung+")");
        }
      if ((welt.object[1].direction+richtung)<-180) {
        welt.object[1].direction=welt.object[1].direction+richtung+360;
        return("(turn "+richtung+")");
        }
      welt.object[1].direction=welt.object[1].direction+richtung;
      return("(turn "+richtung+")");
      }
    double force = 60*(distance(1,0)-0.4*Math.sqrt(welt.object[1].xspeed*welt.object[1].xspeed+welt.object[1].yspeed*welt.object[1].yspeed));
    if ((stamina<300)&&(force>70)) force=70;
    if (force>100) force=100;
    if (Math.abs(richtung)<20) return("(dash "+force+")");
    return("(dash -"+(2*force)+")");
    }

  /** <b>Trgt die Entfernungen der anderen Objekte zum Ball in ein Array ein</b>. */
  public void toBall() {
    int spieler=1;
    nearestPlayer=1;
    nearestTeamPlayer=1;
    ToBall[1]=Math.sqrt((welt.object[1].x-welt.object[0].x)*(welt.object[1].x-welt.object[0].x)+(welt.object[1].y-welt.object[0].y)*(welt.object[1].y-welt.object[0].y));
    while ((spieler<23)&&(welt.object[spieler]!=null)) {
      ToBall[spieler] = Math.sqrt((welt.object[spieler].x-welt.object[0].x)*(welt.object[spieler].x-welt.object[0].x)+(welt.object[spieler].y-welt.object[0].y)*(welt.object[spieler].y-welt.object[0].y));
      if (ToBall[spieler]<ToBall[nearestPlayer]) nearestPlayer=spieler;
      if ((welt.object[spieler].team==1)&&(ToBall[spieler]<ToBall[nearestTeamPlayer])) nearestTeamPlayer=spieler;
      spieler++;
      }
    }

  /**
   * <b>Sucht nach der Nummer im Array des Objects, der am besten anspielbar ist</b>.
   * <br><br>
   * Die Methode sucht nach dem Mitspieler, der weiter in der Gegnerischen hlfte ist,
   * der durch keinen Gegner gedeckt wird und dessen Entfernung zum Spieler minimal ist.
   */
  public int findPlayerToPass() {
    int bestToPass=0;
    int spieler=2;
    while ((spieler<23)&&(welt.object[spieler]!=null)) {
      if ((welt.object[spieler].x>welt.object[0].x+15)&&(welt.object[spieler].team==1)&&(distance(1,spieler)<distance(1,bestToPass))&&(isPassable(spieler))) {
        bestToPass=spieler;
        }
      spieler++;
      }
    return bestToPass;
    }

  /** <b>berprft, ob ein Gegner zwischen dem Spieler und Object[i] steht</b>. */
  public boolean isPassable(int spieler) {
    double angle = direction(1, spieler);
    int counter=2;
    while ((counter<23)&&(welt.object[counter]!=null)) {
      if ((welt.object[counter].team!=1)&&(Math.abs(direction(1, counter)-angle)<criticalAngle)&&(distance(1, spieler)>distance(1,counter))) {
        return false;
        }
      counter++;
      }
    return true;
    }

  /** <b>Liefert die Richtung von <i>MovableObject[i]</i> zu <i>Object[j]</i></b>. */
  public double direction(int i, int j) {
    double decayi=0.4;
    double decayj=0.4;
    if (i==0) decayi=0.94;
    if (j==0) decayj=0.94;
    double richtung=0;
    if ((welt.object[i]==null)||(welt.object[j]==null)) { return 0; }
    if (welt.object[i].x+decayi*welt.object[i].xspeed==welt.object[j].x+decayj*welt.object[j].xspeed) {
      if (welt.object[j].y+decayj*welt.object[j].yspeed>welt.object[i].y+decayi*welt.object[i].yspeed) { richtung=90; }
      else { richtung=-90; }
      }
    if (welt.object[i].x+decayi*welt.object[i].xspeed<welt.object[j].x+decayj*welt.object[j].xspeed) richtung =     Math.toDegrees(Math.atan((welt.object[j].y+decayj*welt.object[j].yspeed-welt.object[i].y-decayi*welt.object[i].yspeed)/(welt.object[j].x+decayj*welt.object[j].xspeed-welt.object[i].x-decayi*welt.object[i].xspeed)));
    if (welt.object[i].x+decayi*welt.object[i].xspeed>welt.object[j].x+decayj*welt.object[j].xspeed) richtung = 180+Math.toDegrees(Math.atan((welt.object[j].y+decayj*welt.object[j].yspeed-welt.object[i].y-decayi*welt.object[i].yspeed)/(welt.object[j].x+decayj*welt.object[j].xspeed-welt.object[i].x-decayi*welt.object[i].xspeed)));
    if (richtung> 180) return(richtung-360);
    if (richtung<-180) return(richtung+360);
    return(richtung);
    }

  /** <b>Liefert die Richtung von <i>MovableOpject[i]</i> zum Punkt (x/y)</b>. */
  public double direction(int i, double x, double y) {
    double decayi=0.4;
    if (i==0) decayi=0.94;
    double richtung=0;
    if (welt.object[i]==null) { return 0; }
    if (welt.object[i].x+decayi*welt.object[i].xspeed==x) {
      if (y>welt.object[i].y+decayi*welt.object[i].yspeed) { return 90; }
      else { return(-90); }
      }
    if (welt.object[i].x+decayi*welt.object[i].xspeed<x) richtung =     Math.toDegrees(Math.atan((y-welt.object[i].y-decayi*welt.object[i].yspeed)/(x-welt.object[i].x-decayi*welt.object[i].xspeed)));
    if (welt.object[i].x+decayi*welt.object[i].xspeed>x) richtung = 180+Math.toDegrees(Math.atan((y-welt.object[i].y-decayi*welt.object[i].yspeed)/(x-welt.object[i].x-decayi*welt.object[i].xspeed)));
    if (richtung> 180) return(richtung-360);
    if (richtung<-180) return(richtung+360);
    return(richtung);
    }

  /** <b>Liefert die Entfernung zwischen <i>MovableObject[i]</i> und <i>MovableObject[j]</i></b>. */
  public double distance(int i, int j) {
    double decayi=0.4;
    double decayj=0.4;
    if (i==0) decayi=0.94;
    if (j==0) decayj=0.94;
    if (welt.object[i]==null || welt.object[j]==null) return 0;
    return Math.sqrt((welt.object[i].x+decayi*welt.object[i].xspeed-welt.object[j].x-decayj*welt.object[j].xspeed)*(welt.object[i].x+decayi*welt.object[i].xspeed-welt.object[j].x-decayj*welt.object[j].xspeed)+(welt.object[i].y+decayi*welt.object[i].yspeed-welt.object[j].y-decayj*welt.object[j].yspeed)*(welt.object[i].y+decayi*welt.object[i].yspeed-welt.object[j].y-decayj*welt.object[j].yspeed));
    }

  /** <b>Liefert die Entfernung zwischen <i>MovableObject[i]</i> und dem Punkt (x/y)</b>. */
  public double distance(int i, double x, double y) {
    double decayi=0.4;
    if (i==0) decayi=0.94;
    if (welt.object[i]==null) return 0;
    return Math.sqrt((welt.object[i].x+decayi*welt.object[i].xspeed-x)*(welt.object[i].x+decayi*welt.object[i].xspeed-x)+(welt.object[i].y+decayi*welt.object[i].yspeed-y)*(welt.object[i].y+decayi*welt.object[i].yspeed-y));
    }
}

