/**
 * Beinhaltet die Entscheidungsalgorythmen eines Torwarts.
 */
public class Torwart extends Spieler {

  /**
   * Zyklus, in dem zuletzt der Catch-Befehl ausgefhrt wurde.
   */
  public int lastCatchCycle;

  int moves;

  /**
   * <b>Konstruktor der Klasse Torwart</b>.
   * <br><br>
   * n=Spielernummer muss 1 sein<br>
   * welt=Weltmodell des Torwarts
   *
   */
  public Torwart(int n, Weltmodell welt) {
    super(n, welt);
    lastCatchCycle=-10;
    moves=0;
    }

  /**
   * <b>Trifft die Entscheidung im laufenden Spiel</b>.
   * <br><br>
   * Ist die Entfernung zum Ball kleiner als 0.5 wird ein Mitspieler
   * gesucht, zu dem gepasst werden kann. Findet sich kein solcher Spieler,
   * dann schiet der Torwart in die Richtung des gegnerischen Tors.
   * <br><br>
   * Ist der Torwart dem Ball am nchsten rennt er Richtung Ball, sonst versucht
   * er den Winkel zwischen Ball und Tor optimal zu verkrzen.
   */
  public String play() {
    // Im Strafraum
    if ((cycle-lastCatchCycle>5)&&(welt.object[0].x<-37)&&(Math.abs(welt.object[0].y)<18)&&(ToBall[1]<1.7)) {
      double dir = direction(1, welt.object[0].x+0.97*welt.object[0].xspeed, welt.object[0].y+0.97*welt.object[0].yspeed)-welt.object[1].direction;
      if (dir>180) { dir=dir-360; }
      if (dir<-180) { dir=dir+360; }
      lastCatchCycle=cycle;
      return("(catch "+dir+")");
      }
    if (ToBall[1]<0.7) {
      int ziel = findPlayerToPass();
      if (ziel != 0) return(Pass(ziel));
      double dir;
      if (welt.object[0].y<0) dir=direction(1, 0, -25)-welt.object[1].direction;
      if (welt.object[0].y>0) dir=direction(1, 0,  25)-welt.object[1].direction;
      dir=-welt.object[1].direction;
      if (dir>180) { dir=dir-360; }
      if (dir<-180) { dir=dir+360; }
      welt.object[0].xspeed=2.7*Math.cos(Math.toRadians(dir));
      welt.object[0].yspeed=2.7*Math.sin(Math.toRadians(dir));
      return("(kick 100 "+dir+")");
      }
    if (distance(1,0)<1.2) {
      double dir=direction(1,0)-welt.object[1].direction;
      if (dir>180) { dir=dir-360; }
      if (dir<-180) { dir=dir+360; }
      if (welt.object[1].direction+dir > 180) {
        welt.object[1].direction=welt.object[1].direction + dir - 360;
        return ("(turn "+dir+")");
        }
      if (welt.object[1].direction+dir <-180) {
        welt.object[1].direction=welt.object[1].direction + dir + 360;
        return ("(turn "+dir+")");
        }
      welt.object[1].direction=welt.object[1].direction + dir;
      return ("(turn "+dir+")");
      }

    // Selbst nchster Spieler
    if (((ToBall[1]<ToBall[nearestTeamPlayer]+2)&&(welt.object[0].x<-25))||((distance(0, 24)<25)&&(welt.object[nearestPlayer].team!=1))) {
      return(moveToBall());
      }
    // Winkel optimieren
//    double innerAngle=Math.toDegrees(Math.acos((distance(0, -52.5, 7)*distance(0, -52.5, 7)+distance(0, -52.5, -7)*distance(0, -52.5, -7)-196)/(2*distance(0, -52.5, 7)*distance(0, -52.5, -7))));
//    double optx=   welt.object[0].x+Math.cos(Math.toRadians(direction(0, -52.5, 7)+innerAngle/2))*4/Math.tan(Math.toRadians(innerAngle/2));
//    double opty=   welt.object[0].y+Math.sin(Math.toRadians(direction(0, -52.5, 7)+innerAngle/2))*4/Math.tan(Math.toRadians(innerAngle/2));
//    if (optx>-47) {
//      opty=opty*10.5/(optx+52.5);
//      optx = -47;
//      }
//    if (optx<-52) {
//      if (opty>7) opty=7;
//      if (opty<-7) opty=-7;
//      optx=-52;
//      }
//    if (opty<-7) {
//      optx = -52.5+(-52.5-optx)*7/opty;
//      opty = -7;
//      }
//    if (opty>7) {
//      optx = -52.5+(optx+52.5)*7/opty;
//      opty = 7;
//      }
//    if (distance(0, optx, opty)>5) {
//      if (distance(1, optx, opty)<2) return(turnToBall());
//      return(moveTo(optx, opty));
//      }
//    return(moveToBall());
 double oy=welt.object[0].y*2/(welt.object[0].x+54.5);
 if (oy>5.5) oy=5.5;
 if (oy<-5.5) oy=-5.5;
 if (distance(1, areaX, oy)<3) return(turnToBall());
 return(moveTo(areaX, oy));
    }

  /**
   * <b>Trifft die Entscheidung wenn die eigene Mannschaft einen Freistoss hat</b>.
   * <br><br>
   * Diese Methode wird auch bei Ecke, Abstoss oder Einwurf angewand.
   *
   */
  public String ownFree() {
    if ((distance(0, areaX, areaY)>15)||(nearestTeamPlayer!=1)) return(moveTo(areaX, areaY));
    if (cycle-netz.FreeKickCycle>25) {
      return(play());
      }
    return(moveToBall());
    }

  /**
   * <b>Trifft die Entscheidung wenn der Gegner einen Freistoss hat</b>.
   * <br><br>
   * In diesem Fall wird nach dem selben Algorythmus wie im <i>play_on</i>-Modus
   * gespielt.<br>
   * Diese Methode wird auch bei Ecke, Abstoss oder Einwurf angewand.
   *
   */
  public String opponentFree() {
    return(moveTo(areaX, areaY));
    }
  }
